package com.jpattern.gwt.client.navigationevent;

import java.util.Map;

import com.google.gwt.core.client.GWT;
import com.google.gwt.core.client.RunAsyncCallback;
import com.jpattern.gwt.client.ApplicationProxy;
import com.jpattern.gwt.client.IApplicationProvider;
import com.jpattern.gwt.client.presenter.IPresenter;

/**
 * 
 * @author Francesco Cina
 *
 * 01/ago/2011
 */
public abstract class ANavigationEvent implements INavigationEvent {

	private IApplicationProvider provider;
	private final String eventName;
	private boolean singletonPresenter = false;
	private IPresenter presenter;
	private boolean runAsync = false;

	public ANavigationEvent(String eventName) {
		this(eventName, ApplicationProxy.getInstance().getApplicationProvider());
	}
	
	public ANavigationEvent(String eventName, IApplicationProvider provider) {
		this.eventName = eventName;
		this.provider = provider;
	}
	
	@Override
	public final void notifyNavigationEvent(IPresenter parentPresenter, Map<String, String> queryStringValues, boolean registerEvent) {
		getProvider().getHistoryService().getHistoryManager().registerHistory(parentPresenter, queryStringValues, registerEvent, this);
	}
	
	@Override
	public IPresenter launch(IPresenter parentPresenter, Map<String, String> queryStringValues, String childInitialEventName) {
		IPresenter presenter = getPresenter(queryStringValues);
		presenter.ignoreInitialEventWithSameTargetOf(childInitialEventName);
		initPresenter(parentPresenter, presenter);
		return presenter;
	}
	
	@Override
	public IPresenter launch(IPresenter parentPresenter, Map<String, String> queryStringValues) {
		IPresenter presenter = getPresenter(queryStringValues);
		initPresenter(parentPresenter, presenter);
		return presenter;
	}
	
	private void initPresenter(IPresenter parentPresenter, IPresenter presenter) {
		presenter.setParent(parentPresenter);
		presenter.initPresenter();
		parentPresenter.getView().onNavigationEvent(getName());
	}
	
	private IPresenter getPresenter(Map<String, String> queryStringValues) {
		if (presenter==null || !singletonPresenter) {
			presenter = exec(queryStringValues);
		}
		return presenter;
	}

	/**
	 * Return the newly created presenter
	 * @return
	 */
	public abstract IPresenter exec(Map<String, String> queryStringValues); 
	
	protected IApplicationProvider getProvider() {
		if (provider == null) {
			provider = ApplicationProxy.getInstance().getApplicationProvider();
		}
		return provider;
	}
	
	public final String getName() {
		return eventName;
	}
	
	@Override
	public boolean getSingletonPresenter() {
		return singletonPresenter;
	}

	@Override
	public void setSingletonPresenter(boolean singletonPresenter) {
		this.singletonPresenter = singletonPresenter;
	}
	
	public void run() {
		if (runAsync) {
			GWT.runAsync(new RunAsyncCallback() {

				@Override
				public void onSuccess() {
					
				}

				@Override
				public void onFailure(Throwable reason) {
				}
			});
		} else {

		}
	}
}
